@@ -2,7 +2,7 @@ package Net::OAuth::Simple;
 
 use warnings;
 use strict;
-our $VERSION = "1.4";
+our $VERSION = "1.5";
 
 use URI;
 use LWP;
@@ -10,6 +10,7 @@ use CGI;
 use HTTP::Request::Common ();
 use Carp;
 use Net::OAuth;
+use Scalar::Util qw(blessed);
 require Net::OAuth::Request;
 require Net::OAuth::RequestTokenRequest;
 require Net::OAuth::AccessTokenRequest;
@@ -154,6 +155,12 @@ If you pass in a key C<protocol_version> with a value equal to B<1.0a> then
 the newest version of the OAuth protocol will be used. A value equal to B<1.0> will 
 mean the old version will be used. Defaults to B<1.0a>
 
+You can pass in your own User Agent by using the key C<browser>.
+
+If you pass in C<return_undef_on_error> then instead of C<die>-ing on error  
+methods will return undef instead and the error can be retrieved using the 
+C<last_error()> method. See the section on B<ERROR HANDLING>.
+
 =cut
 
 sub new {
@@ -161,12 +168,14 @@ sub new {
     my %params = @_;
     $params{protocol_version} ||= '1.0a';
     my $client = bless \%params, $class;
+    
+    # Set up LibWWWPerl for HTTP requests
+    $client->{browser} ||= LWP::UserAgent->new;
 
     # Verify arguments
     $client->_check;
 
-    # Set up LibWWWPerl for HTTP requests
-    $client->{browser} = LWP::UserAgent->new;
+   
 
     # Client Object
     return $client;
@@ -175,15 +184,15 @@ sub new {
 # Validate required constructor params
 sub _check {
     my $self = shift;
-    use Data::Dumper;
-    #die Dumper ($self);
-
 
     foreach my $param ( @required_constructor_params ) {
         unless ( defined $self->{tokens}->{$param} ) {
-            die "Missing required parameter '$param'";
+            return $self->_error("Missing required parameter '$param'");
         }
     }
+    
+    return $self->_error("browser must be a LWP::UserAgent")
+        unless blessed $self->{browser} && $self->{browser}->isa('LWP::UserAgent');
 }
 
 =head2 oauth_1_0a 
@@ -419,8 +428,6 @@ sub _store {
     my $key  = shift;
     $self->{$ns}->{$key} = shift if @_;
     return $self->{$ns}->{$key};
-
-    
 }
 
 =head2 authorization_url
@@ -478,7 +485,7 @@ C<get_authorization_url> first.
 
 Returns the access token and access token secret but also sets
 them internally so that after calling this method you can
-immediately call C<location> or C<update_location>.
+immediately call a restricted method.
 
 If you pass in a hash of params then they will added as parameters to the URL.
 
@@ -494,7 +501,7 @@ sub request_access_token {
     
     if ($self->oauth_1_0a) {
         $params{verifier} = $self->verifier                             unless defined $params{verifier};
-        die "You must pass a verified parameter when using OAuth v1.0a" unless defined $params{verifier};
+        return $self->_error("You must pass a verified parameter when using OAuth v1.0a") unless defined $params{verifier};
         
     }
     
@@ -523,7 +530,7 @@ sub _decode_tokens {
 
     delete $self->{tokens}->{$_} for qw(request_token request_token_secret verifier);
 
-    die "ERROR: $url did not reply with an access token"
+    return $self->_error("ERROR: $url did not reply with an access token")
       unless ( $self->access_token && $self->access_token_secret );
 
     return ( $self->access_token, $self->access_token_secret );
@@ -553,13 +560,12 @@ sub xauth_request_access_token {
     my $self = shift;
     my %params = @_;
     my $url = $self->access_token_url;
-	$url =~ s !^http:!https:!; # force https
+    $url =~ s !^http:!https:!; # force https
 
     my %xauth_params = map { $_ => $params{$_} } 
         grep {/^x_auth_/}
         @{Net::OAuth::XauthAccessTokenRequest->required_message_params};
 
-	use Data::Dumper;
     my $access_token_response = $self->_make_request(
         'Net::OAuth::XauthAccessTokenRequest',
         $url, 'POST',
@@ -587,7 +593,7 @@ sub request_request_token {
     
     if ($self->oauth_1_0a) {
         $params{callback} = $self->callback                             unless defined $params{callback};
-        die "You must pass a callback parameter when using OAuth v1.0a" unless defined $params{callback};
+        return $self->_error("You must pass a callback parameter when using OAuth v1.0a") unless defined $params{callback};
     }
           
     my $request_token_response = $self->_make_request(
@@ -595,7 +601,7 @@ sub request_request_token {
         $url, 'GET', 
         %params);
 
-    die "GET for $url failed: ".$request_token_response->status_line
+    return $self->_error("GET for $url failed: ".$request_token_response->status_line)
       unless ( $request_token_response->is_success );
 
     # Cast response into CGI query for EZ parameter decoding
@@ -607,7 +613,7 @@ sub request_request_token {
     $self->request_token_secret($request_token_response_query->param('oauth_token_secret'));
     $self->callback_confirmed($request_token_response_query->param('oauth_callback_confirmed'));
     
-    die "Response does not confirm to OAuth1.0a. oauth_callback_confirmed not received"
+    return $self->_error("Response does not confirm to OAuth1.0a. oauth_callback_confirmed not received")
      if $self->oauth_1_0a && !$self->callback_confirmed;
 
 }
@@ -642,7 +648,7 @@ Any extra parameters can be passed in as a hash.
 sub make_restricted_request {
     my $self     = shift;
 
-    croak $UNAUTHORIZED unless $self->authorized;
+    return $self->_error($UNAUTHORIZED) unless $self->authorized;
 
     return $self->_restricted_request( $self->access_token, $self->access_token_secret, @_ );
 }
@@ -682,48 +688,74 @@ sub _restricted_request {
 
 sub _make_request {
     my $self    = shift;
-
     my $class   = shift;
     my $url     = shift;
-    my $method  = lc(shift);
-    my %extra   = @_;
+    my $method  = uc(shift);
+    my @extra   = @_;
 
     my $uri   = URI->new($url);
     my %query = $uri->query_form;
     $uri->query_form({});
-
+    
     my $request = $class->new(
         consumer_key     => $self->consumer_key,
         consumer_secret  => $self->consumer_secret,
         request_url      => $uri,
-        request_method   => uc($method),
+        request_method   => $method,
         signature_method => $self->signature_method,
         protocol_version => $self->oauth_1_0a ? Net::OAuth::PROTOCOL_VERSION_1_0A : Net::OAuth::PROTOCOL_VERSION_1_0,
         timestamp        => time,
         nonce            => $self->_nonce,
         extra_params     => \%query,
-        %extra,
+        @extra,
     );
     $request->sign;
-    die "COULDN'T VERIFY! Check OAuth parameters.\n"
+    return $self->_error("COULDN'T VERIFY! Check OAuth parameters.")
       unless $request->verify;
 
-    my $params = $request->to_hash;
-     my $req;
-    if ($method eq 'post') {
-         $req = HTTP::Request::Common::POST($uri, Content => $params);
+    my @args    = ();
+    my $req_url = $url;
+    my $params  = $request->to_hash;
+    if ('GET' eq $method || 'PUT' eq $method) {
+        $req_url = URI->new($url);
+        $req_url->query_form(%$params);
     } else {
-         my $request_url = URI->new($url);
-        $request_url->query_form(%$params);
-        $req = HTTP::Request::Common::GET($request_url);
+        @args    = ( HTTP::Headers->new(%$params) );
     }
+    
+    my $req      = HTTP::Request->new( $method => $req_url, @args);
     my $response = $self->{browser}->request($req);
-    die "$method on $request failed: ".$response->status_line
+    return $self->_error("$method on $request failed: ".$response->status_line)
       unless ( $response->is_success );
 
     return $response;
 }
 
+sub _error {
+    my $self = shift;
+    my $mess = shift;
+    if ($self->{return_undef_on_error}) {
+        $self->{_last_error} = $mess;
+    } else {
+        croak $mess;
+    }
+    return undef;
+}
+
+=head2 last_error
+
+Get the last error message.
+
+Only works if C<return_undef_on_error> was passed in to the constructor.
+
+See the section on B<ERROR HANDLING>.
+
+=cut
+sub last_error {
+    my $self = shift;
+    return $self->{_last_error};
+}
+
 =head2 load_tokens <file>
 
 A convenience method for loading tokens from a config file.
@@ -777,6 +809,19 @@ sub save_tokens {
     close($fh);
 }
 
+=head1 ERROR HANDLING
+
+Originally this module would die upon encountering an error (inheriting behaviour 
+from the original Yahoo! code).
+
+This is still the default behaviour however if you now pass 
+
+    return_undef_on_error => 1
+    
+into the constructor then all methods will return undef on error instead.
+
+The error message is accessible via the C<last_error()> method.
+
 =head1 GOOGLE'S SCOPE PARAMETER
 
 Google's OAuth API requires the non-standard C<scope> parameter to be set 
@@ -808,7 +853,7 @@ Here's an example class that uses Google's Portable Contacts API via OAuth:
     my $oauth = Net::AppUsingGoogleOAuth->new(%tokens);
 
     # Web application
-    $app->redirect( $oauth->get_authorization_url(oauth_callback => "http://you.example.com/oauth/callback") );
+    $app->redirect( $oauth->get_authorization_url( callback => "http://you.example.com/oauth/callback") );
 
     # Desktop application
     print "Open the URL and come back once you're authenticated!\n",
@@ -822,6 +867,11 @@ services API documentation for the possible list of I<scope> parameter value.
 If C<Math::Random::MT> is installed then any nonces generated will use a 
 Mersenne Twiser instead of Perl's built in randomness function.
 
+=head1 EXAMPLES
+
+There are example Twitter and Twitter xAuth 'desktop' apps and a FireEagle OAuth 1.0a web app 
+in the examples directory of the distribution.
+
 =head1 BUGS
 
 Non known
